/*
 * Copyright 2014-2015 ISP RAS (http://www.ispras.ru)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase;

import java.util.Iterator;
import java.util.NoSuchElementException;

import ru.ispras.fortress.util.InvariantChecks;

public abstract class TestDataProvider implements Iterator<TestData> {

  private static final TestDataProvider EMPTY_PROVIDER = new TestDataProvider() {
    @Override
    public boolean hasNext() {
      return false;
    }

    @Override
    public TestData next() {
      throw new NoSuchElementException();
    }
  };

  @Override
  public void remove() {
    throw new UnsupportedOperationException(
        "TestDataProvider: remove() method is no supported");
  }

  private static final class Repeater extends TestDataProvider {
    private final TestData data;
    private int ntimes;

    public Repeater(final TestData data, int ntimes) {
      InvariantChecks.checkNotNull(data);
      InvariantChecks.checkTrue(ntimes > 0);

      this.data = data;
      this.ntimes = ntimes;
    }

    @Override
    public boolean hasNext() {
      return ntimes > 0;
    }

    @Override
    public TestData next() {
      if (!hasNext()) {
        throw new NoSuchElementException();
      }
      --ntimes;
      return data;
    }
  }

  public static TestDataProvider empty() {
    return EMPTY_PROVIDER;
  }

  public static TestDataProvider singleton(final TestData data) {
    return new Repeater(data, 1);
  }

  public static TestDataProvider repeat(final TestData data, int n) {
    return new Repeater(data, n);
  }

  public abstract boolean hasNext();
  public abstract TestData next();
}
